#include "lptimer32.h"

void LPTIM_IRQHandler(void)
{
    /* enter interrupt */
    rt_interrupt_enter();

    if(FL_LPTIM32_IsEnabledIT_Update(LPTIM32) && FL_LPTIM32_IsActiveFlag_Update(LPTIM32))
    {
        FL_LPTIM32_ClearFlag_Update(LPTIM32);
    }
    /* leave interrupt */
    rt_interrupt_leave();
}


/**
 * This function get current count value of LPTIM
 *
 * @return the count vlaue
 */
rt_uint32_t fm33xx_lptimer_get_current_tick(void)
{
    if (FL_LPTIM32_ReadCounter(LPTIM32))
        return FL_LPTIM32_ReadCounter(LPTIM32);
    else
        return FL_LPTIM32_ReadAutoReload(LPTIM32);
}

/**
 * This function get the max value that LPTIM can count
 *
 * @return the max count
 */
rt_uint32_t fm33xx_lptimer_get_tick_max(void)
{
    return (0xFFFFFFFF);
}

/**
 * This function start LPTIM with reload value
 *
 * @param reload The value that LPTIM count down from
 *
 * @return RT_EOK
 */
rt_err_t fm33xx_lptimer_start(rt_uint32_t reload)
{
    FL_LPTIM32_EnableIT_Update(LPTIM32 );
    FL_LPTIM32_WriteAutoReload(LPTIM32, reload);
    FL_LPTIM32_Enable(LPTIM32);

    return (RT_EOK);
}

/**
 * This function stop LPTIM
 */
void fm33xx_lptimer_stop(void)
{
    FL_LPTIM32_Disable(LPTIM32);
    FL_LPTIM32_ClearFlag_Update(LPTIM32);
    FL_LPTIM32_DisableIT_Update(LPTIM32 );
}

/**
 * This function get the count clock of LPTIM
 *
 * @return the count clock frequency in Hz
 */
rt_uint32_t fm33xx_lptimer_get_countfreq(void)
{
    return 32768;
}

/**
 * This function initialize the lptim
 */
int fm33xx_hw_lptimer_init(void)
{

    FL_LPTIM32_InitTypeDef    defaultInitStruct;
    NVIC_ConfigTypeDef    InterruptConfigStruct;

    defaultInitStruct.clockSource = FL_RCC_LPTIM32_CLK_SOURCE_LSCLK;
    defaultInitStruct.prescalerClockSource = FL_LPTIM32_CLK_SOURCE_INTERNAL;
    defaultInitStruct.prescaler = FL_LPTIM32_PSC_DIV1;
    defaultInitStruct.autoReload = 32767;
    defaultInitStruct.mode = FL_LPTIM32_OPERATION_MODE_NORMAL;
    defaultInitStruct.onePulseMode = FL_LPTIM32_ONE_PULSE_MODE_CONTINUOUS;
    defaultInitStruct.countEdge = FL_LPTIM32_ETR_COUNT_EDGE_RISING;
    defaultInitStruct.triggerEdge = FL_LPTIM32_ETR_TRIGGER_EDGE_RISING;

    FL_LPTIM32_Init(LPTIM32,&defaultInitStruct );

    FL_LPTIM32_EnableIT_Update(LPTIM32 );
    
    InterruptConfigStruct.preemptPriority = 0x02;
    NVIC_Init(&InterruptConfigStruct,LPTIM_IRQn );
    
    FL_LPTIM32_Enable(LPTIM32);
    return 0;
}
